@@ -60,6 +60,22 @@ class AuthDecorator(object):
     Note that since it uses flask.session, you'll need to set
     :attr:`app.secret_key`.
 
+    We need to be able to reliably determine the hostname of the current
+    website. This is retrieved from :attr:`flask.Request.url`.
+    By default, Werkzeug will respect the value of a ``X-Forwarded-Host``
+    header, which means that a man-in-the-middle can have someone authenticate
+    to *their* website, and forward the response from the WLS on to you.
+    You must either set :attr:`flask.Request.trusted_hosts`, for example
+    like so::
+
+        class R(flask.Request):
+            trusted_hosts = {'www.danielrichman.co.uk'}
+        app.request_class = R
+
+    ... or sanitise both the `Host` header *and* the `X-Forwarded-Host`
+    header in your web-server. If you choose the second option, set
+    `can_trust_request_host`.
+
     This tries to emulate the feel of applying mod_ucam_webauth to a file.
 
     The decorator wraps the view in a function that calls
@@ -106,6 +122,10 @@ class AuthDecorator(object):
     :type require_ptags: :class:`set` of :class:`str`, or ``None``
     :param require_ptags: require the ptags to contain `any` string in
                           `require_ptags` (i.e., non empty intersection)
+    :type can_trust_request_host: :class:`bool`
+    :param can_trust_request_host: Can we trust the hostname in
+                                   ``request.url``?
+                                   (see :ref:`checking-response-values`)
 
     More complex customisation is possible:
 
@@ -149,7 +169,8 @@ class AuthDecorator(object):
                     max_life=7200, use_wls_life=False,
                     inactive_timeout=None, issue_bounds=(15,5),
                     require_principal=None,
-                    require_ptags=frozenset(["current"])):
+                    require_ptags=frozenset(["current"]),
+                    can_trust_request_host=False):
         self.desc = desc
         self.aauth = aauth
         self.iact = iact
@@ -160,6 +181,7 @@ class AuthDecorator(object):
         self.issue_bounds = issue_bounds
         self.require_principal = require_principal
         self.require_ptags = require_ptags
+        self.can_trust_request_host = can_trust_request_host
 
     def __call__(self, view_function):
         """
@@ -185,18 +207,38 @@ class AuthDecorator(object):
     def _state(self):
         return session.get("_ucam_webauth", {}).get("state", {})
 
-    def _prop_helper(self, name):
-        return self._state.get(name, None)
+    @_state.setter
+    def _state(self, new):
+        if "_ucam_webauth" not in session:
+            session["_ucam_webauth"] = {}
+        session["_ucam_webauth"]["state"] = new
+
+    @_state.deleter
+    def _state(self):
+        if "_ucam_webauth" in session:
+            d = session["_ucam_webauth"]
+            if "state" in d:
+                del d["state"]
+
+    @property
+    def _params_token(self):
+        return session.get("_ucam_webauth", {}).get("params_token", None)
+
+    @_params_token.setter
+    def _params_token(self, new):
+        if "_ucam_webauth" not in session:
+            session["_ucam_webauth"] = {}
+        session["_ucam_webauth"]["params_token"] = new
 
     @property
     def principal(self):
         """The current principal, or ``None``"""
-        return self._prop_helper("principal")
+        return self._state.get("principal")
 
     @property
     def ptags(self):
         """The current ptags, or ``None``"""
-        ptags = self._prop_helper("ptags")
+        ptags = self._state.get("ptags")
         if ptags is not None:
             ptags = frozenset(ptags)
         return ptags
@@ -210,17 +252,17 @@ class AuthDecorator(object):
         the :class:`datetime` object used by :class:`ucam_webauth.Response`.
         (`issue` is ``None`` if there is no current session.)
         """
-        return self._prop_helper("issue")
+        return self._state.get("issue")
 
     @property
     def life(self):
         """life of the last WLS response (:class:`int` seconds), or ``None``"""
-        return self._prop_helper("life")
+        return self._state.get("life")
 
     @property
     def last(self):
         """Time (:class:`int` unix timestamp) of the last decorated request"""
-        return self._prop_helper("last")
+        return self._state.get("last")
 
     @property
     def expires(self):
@@ -250,9 +292,8 @@ class AuthDecorator(object):
 
     def logout(self):
         """Clear the auth., and return a redirect to the WLS' logout page"""
-        if self._state != {}:
-            session.modified = True
-            del session["_ucam_webauth"]["state"]
+        session.modified = True
+        del self._state
         return redirect(self.logout_url, code=303)
 
     def before_request(self):
@@ -262,17 +303,13 @@ class AuthDecorator(object):
         * checks if there is a response from the WLS
 
           * checks if the current URL matches that which the WLS said it
-            redirected to (the intention is to avoid an evil admin of another
-            site replaying successful authentications to us, but it isn't
-            perfect unless you're careful with your `Host` and
-            `X-Forwarded-Host` headers)
+            redirected to (avoid an evil admin of another site replaying
+            successful authentications)
           * checks if ``flask.session`` is empty - if so, then we deduce that
             the user has cookies disabled, and must abort immediately with
             403 Forbidden, or we will start a redirect loop
           * checks if `params` matches the token we set (and saved in
             ``flask.session``) when redirecting to Raven
-            (preventing replaying successful authentications from other
-            sites onto ours)
           * checks if the authentication method used is permitted by `aauth`
             and user-interaction respected `iact` - if not, abort with
             400 Bad Request
@@ -365,7 +402,7 @@ class AuthDecorator(object):
         if url_without_response is None:
             abort(400)
 
-        token = session.get("_ucam_webauth", {}).get("params_token")
+        token = self._params_token
         if token is None or response.params != token:
             logger.warning("params token mismatch: session=%s response=%s",
                            token, response.params)
@@ -387,14 +424,14 @@ class AuthDecorator(object):
                 logger.debug("new session")
                 self.session_new()
 
-            session["_ucam_webauth"]["state"] = \
+            self._state = \
                     {"principal": response.principal,
                      "ptags": list(response.ptags),
                      "issue": issue, "life": response.life,
                      "last": time()}
 
         else:
-            session["_ucam_webauth"]["state"] = {"response_failure": True}
+            self._state = {"response_failure": True}
 
         return redirect(url_without_response, code=303)
 
@@ -408,6 +445,11 @@ class AuthDecorator(object):
         redirected after authentication.
         """
 
+        if not self.can_trust_request_host and request.trusted_hosts is None:
+            raise RuntimeError("Either set request.trusted_hosts, or sanitise "
+                               "Host/X-Forwarded-Host headers and pass "
+                               "can_trust_request_host = True")
+
         actual_url = request.url
 
         # note: mod_ucam_webauth simply strips everything up to a ?
@@ -470,17 +512,13 @@ class AuthDecorator(object):
 
     def _redirect_to_wls(self):
         """Create a request and return a redirect to the WLS"""
-        if "_ucam_webauth" not in session:
-            session["_ucam_webauth"] = {}
-        if "params_token" not in session["_ucam_webauth"]:
-            session["_ucam_webauth"]["params_token"] = \
+        if self._params_token is None:
+            self._params_token = \
                     base64.b64encode(os.urandom(18)).decode("ascii")
 
-        token = session["_ucam_webauth"]["params_token"]
-
         req = self.request_class(url=request.url, desc=self.desc,
                                  aauth=self.aauth, iact=self.iact,
-                                 msg=self.msg, params=token)
+                                 msg=self.msg, params=self._params_token)
         return redirect(str(req), code=303)
 
     def _get_expires(self, state):
